//
//  PhobosDataManager.m
//  GMCache
//
//  Created by Locus on 2020/1/21.
//

#import "PhobosDataManager.h"
#import "PhobosSendManager.h"
#import "PhobosConfig.h"
#import "PhobosUtil.h"
#import "GMSQliteManager.h"
#import "GMPhobosSqulitModel.h"
#import <MJExtension/MJExtension.h>
#import "GMPhobosThread.h"

#define KtableName @"phobosTable"

@interface PhobosDataManager ()
//数据库操作对象（单利）
@property (nonatomic, strong) GMSQliteManager *squlitManager;
/// 记录没有发送的数据个数（大于50 就要发送数据）
@property (nonatomic, assign) NSInteger messageCont;
/// runloop线程，用来处理埋点的数据
@property (nonatomic, strong) GMPhobosThread *phobosThread;
@end

@implementation PhobosDataManager

static PhobosDataManager *dataManager;

+ (PhobosDataManager *)sharedPhobosDataManager {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        dataManager = [[PhobosDataManager alloc] init];
    });
    return dataManager;
}

- (instancetype)init {
    self = [super init];
    if (self) {
        [self setupSDK];
    }
    return self;
}

- (void)removeAll {
    [self performSelector:@selector(remove) onThread:_phobosThread withObject:nil waitUntilDone:YES];
}

- (void)remove {
    bool status = [_squlitManager gm_deleteAllDataFromTable:KtableName];
    NSLog(@"清空表格数状态:%d",status);
}

- (int)messageCount {
    int count = [_squlitManager gm_tableItemCount:KtableName whereFormat:nil];
    NSLog(@"数据总数：%d",count);
    return count;
}

#pragma mark - 初始化
- (void)setupSDK {
    _squlitManager = [GMSQliteManager sharedSQliteManager];
    //创建表格
    bool status = [_squlitManager gm_createTable:KtableName model:[GMPhobosSqulitModel class]];
    //初始化线程和runloop
    __weak typeof(self) weakSelf = self;
    _phobosThread = [[GMPhobosThread alloc] initWithBlock:^{
        [[NSRunLoop currentRunLoop] addPort:[NSPort new] forMode:NSDefaultRunLoopMode];
        while (weakSelf) {
            [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
        }
    }];
    _phobosThread.name = @"phobosThread";
    [_phobosThread start];
    //修改数据
    [self performSelector:@selector(resetData) onThread:_phobosThread withObject:nil waitUntilDone:NO];
}

#pragma mark - 方法只用在APP启动的时候
- (void)resetData {
    /** 将上次没有获取到发送结果的数据的状态修改为发送失败，待下次重新发送 */
    NSString *whereStr = [NSString stringWithFormat:@"where status = %d", PhobosDataSendStatus_send];
    [_squlitManager gm_updateTable:KtableName dicOrModel:@{@"status":@(PhobosDataSendStatus_error)} whereFormat:whereStr];
    /** 将发送成功的数据删除 */
    NSString *whereDelet = [NSString stringWithFormat:@"where status = %d", PhobosDataSendStatus_finish];
    [_squlitManager gm_deleteTable:KtableName whereFormat:whereDelet];
    /** 设置未发送数据数量 */
    NSString *whereCount = [NSString stringWithFormat:@"where status = %d or status = %d", PhobosDataSendStatus_wait, PhobosDataSendStatus_error];
    _messageCont = [_squlitManager gm_tableItemCount:KtableName whereFormat:whereCount];
}

#pragma mark - 插入数据
- (void)insertData:(NSDictionary *)data
           sendAPI:(NSString *)sendAPI
        phobosType:(NSString *)phoboType
       immediately:(BOOL)status {
    if (!sendAPI || [sendAPI isEqualToString:@""] || !data) return;
    GMPhobosSqulitModel *squlitModel = [GMPhobosSqulitModel modelOfDict:data type:phoboType urlApi:sendAPI immediately:status];
    [self performSelector:@selector(insertModel:) onThread:_phobosThread withObject:squlitModel waitUntilDone:NO];
}

- (void)insertModel:(GMPhobosSqulitModel *)squlitModel {
    BOOL insertType = [_squlitManager gm_insertTable:KtableName model:squlitModel];
    if (!insertType) {//如果插入失败，就直接发送给服务器
        [self sendModelArray:@[squlitModel]];
    } else if (squlitModel.immediately) {//需要立刻发送的埋点
        [self disposeSendDataWithImmediately:YES];
    } else {//插入到数据中
        self.messageCont++;
        [self disposeSendDataWithImmediately:NO];
    }
    NSLog(@"插入数状态%d count:%d", insertType,_messageCont);
}

#pragma mark - 获取待发送数据并修改状态为“发送”
- (NSArray *)fetchToBeSendDataEntitiesAndUpdateWithSendStatus {
    NSString *where = [NSString stringWithFormat:@"where status = %d or status = %d", PhobosDataSendStatus_wait, PhobosDataSendStatus_error];
    /**获取待发送的数据*/
    NSArray *array = [_squlitManager gm_lookupTable:KtableName filterArray:nil objectClass:[GMPhobosSqulitModel class] whereFormat:where];
    /**将待发送的数据修改为“发送中”*/
    [_squlitManager gm_updateTable:KtableName dicOrModel:@{@"status":@(PhobosDataSendStatus_send)} whereFormat:where];
    return array;
}

#pragma mark - 是否需要发送数据
- (void)disposeSendDataWithImmediately:(BOOL)status {
    NSString *where = [NSString stringWithFormat:@"where status = %d or status = %d", PhobosDataSendStatus_wait, PhobosDataSendStatus_error];
    _messageCont = [_squlitManager gm_tableItemCount:KtableName whereFormat:where];
    if (status || _messageCont >= PhobosShardCount) {
        NSArray *array = [self fetchToBeSendDataEntitiesAndUpdateWithSendStatus];
        NSMutableArray *entities = [GMPhobosSqulitModel mj_objectArrayWithKeyValuesArray:array];
        [self sendModelArray:entities];
    }
}

- (void)sendModelArray:(NSArray<GMPhobosSqulitModel *> *)entities {
    NSLog(@"发送*******%lld", entities.count);
    [PhobosSendManager sendDataWithEntities:entities completion:^(NSArray<GMPhobosSqulitModel *> *finishEntities, NSInteger code) {
        SEL selelct = code == 200 ?  @selector(sendSuccess:) :@selector(sendError:);
        [self performSelector:selelct onThread:_phobosThread withObject:finishEntities waitUntilDone:NO];
    }];
}

#pragma mark - 上报数据成功
- (void)sendSuccess:(NSArray<GMPhobosSqulitModel*> *)entities {
    for (GMPhobosSqulitModel *model in entities) {
        NSString *whereStr = [NSString stringWithFormat:@"where messageId = '%@'", model.messageId];
        bool status = [_squlitManager gm_deleteTable:KtableName whereFormat:whereStr];
        NSLog(@"删除状态：%d", status);
    }
}

#pragma mark - 上报数据失败
- (void)sendError:(NSArray<GMPhobosSqulitModel*> *)entities {
    for (GMPhobosSqulitModel *model in entities) {
        NSString *whereStr = [NSString stringWithFormat:@"where messageId = '%@'", model.messageId];
        bool status = [_squlitManager gm_updateTable:KtableName
                                          dicOrModel: @{@"status":@(PhobosDataSendStatus_error)}
                                         whereFormat:whereStr];
        NSLog(@"发送失败后修改状态:%d",status);
    }
}

@end
